let counter = 0;

const signBuffer = buffer => {
    const bLength = buffer.length;
    let signedBuffer = Buffer.allocUnsafe(bLength + 1);
    buffer.copy(signedBuffer);
    signedBuffer[bLength] = signedBuffer[0];
    for (let i = 1; i < bLength; i++) {
      signedBuffer[bLength] ^= signedBuffer[i];
    }
    return signedBuffer;
};

const convertBufferToBootloaderVersion = (dataBuf) => {
    const bootloaderStringArr = dataBuf.toString('utf8',0,5).replace(/\0/g, '').split('.')

    let version = {major: 0,minor: 0, patch: 0}
    let canUpdateFirmware = false;

    if(!bootloaderStringArr[0].toLowerCase().includes("null")) {
       version.major = Number(bootloaderStringArr[0])
       version.minor = Number(bootloaderStringArr[1])

       canUpdateFirmware = (version.major >= 4 && version.minor >= 1) ? true : false
    }


    const bootloader =  {version,canUpdateFirmware}

    return bootloader
}

const buildPacket = (buff, index, chunk) => {

    let buffer = Buffer.from(buff);
    let chunkBuffer = Buffer.from(chunk)

    buffer.writeUInt16BE(index,6)
    buffer.fill(chunkBuffer,8)

    return buffer;
}

const WIFI_GET_BOOTLOADER_VERSION = 0x21;
const FIRMWARE_UPDATE_BOOT_COMMAND = 0xFD
const FIRMWARE_UPDATE_START_COMMAND = 0x5A
const FIRMWARE_UPDATE_END_COMMAND = 0xA5
const FIRMWARE_UPDATE_PACKET_COMMAND = 0x55
const FIRMWARE_UPDATE_RESET_COMMAND = 0xAA

const commandBuffers = {
    getBootloaderVersionSigned: signBuffer(Buffer.from([0xd0,0xf1,0,6,WIFI_GET_BOOTLOADER_VERSION])),
    firmwareUpdateBootSigned: signBuffer(Buffer.from([0xd0,0xf1,0,6,FIRMWARE_UPDATE_BOOT_COMMAND])),
    firmwareUpdateResetSigned: signBuffer(Buffer.from([0xd0,0xf1,0,7,0x55,FIRMWARE_UPDATE_RESET_COMMAND])),
    firmwareUpdateStartSigned: signBuffer(Buffer.from([0xd0,0xf1,0,7,0xa5,FIRMWARE_UPDATE_START_COMMAND])),
    firmwareUpdateEndSigned: signBuffer(Buffer.from([0xd0,0xf1,0,7,0x5a,FIRMWARE_UPDATE_END_COMMAND])),
    firmwareUpdatePacket: Buffer.alloc(136).fill(Buffer.from([0xd0,0xf1,0,0x89,0xAA,FIRMWARE_UPDATE_PACKET_COMMAND]))
}

const getBootloaderVersion = (connected,socket) => {return connected && socket.write(commandBuffers.getBootloaderVersionSigned)};
const sendBootCommand = (connected,socket) => {return connected && socket.write(commandBuffers.firmwareUpdateBootSigned)}
const sendUpdateResetCommand = (connected,socket) => {return connected && socket.write(commandBuffers.firmwareUpdateResetSigned)}
const sendUpdateStartCommand = (connected,socket) => {console.log({firmwareUpdateStartSigned: commandBuffers.firmwareUpdateStartSigned}); return connected && socket.write(commandBuffers.firmwareUpdateStartSigned)}
const sendUpdateEndCommand = (connected,socket) => { console.log({firmwareUpdateEndSigned: commandBuffers.firmwareUpdateEndSigned}); return connected && socket.write(commandBuffers.firmwareUpdateEndSigned)}
const sendPacketCommand = (connected,socket,index, chunk, reportStatus) => {

    const packet = buildPacket(commandBuffers.firmwareUpdatePacket,index,chunk)
    const buf = signBuffer(packet)
    if(reportStatus) reportStatus(((counter/797)*100).toFixed(2))
    //process.stdout.write(((counter/797)*100).toFixed(2) + "% \r")
    counter++
    return connected && socket.write(buf)

}

module.exports = {
    getBootloaderVersion,
    sendBootCommand,
    sendUpdateResetCommand,
    sendPacketCommand,
    sendUpdateStartCommand,
    sendUpdateEndCommand,
    converters: {
        convertBufferToBootloaderVersion
    },
    costants: {
        WIFI_GET_BOOTLOADER_VERSION,
        FIRMWARE_UPDATE_BOOT_COMMAND
    }
}
