const SerialPort = require('serialport');
const ByteLength = require('@serialport/parser-byte-length')
const path = require('path')
const {startElectron} = require('./electron')
const {io} = require('./socket')
let frontendSocket = null
let canUpdate = false
let faseAggiornamento = "1/3"
let oldFaseAggiornamento = ""
startElectron()

io.of('/frontend').on("connection", socket =>
{
  frontendSocket = socket;

  socket.on('start_update', (data) => {

    if(canUpdate) {
      setTimeout(() => {
         sendBootCommand(true,PORT1);
         bootRequested = true;
      },2000)
    }

  })

  socket.on('ready', () => {

    socket.emit('update_allowed',{canUpdate})
  })

});

const reportStatus = (status) => {
  if(frontendSocket) frontendSocket.emit('status', status)
  if(frontendSocket && (faseAggiornamento !== oldFaseAggiornamento)) {
    frontendSocket.emit('fase', faseAggiornamento)
    oldFaseAggiornamento = faseAggiornamento
  }
}

const {costants,converters, getBootloaderVersion, sendBootCommand } = require('./firmware/usb')
const {getFirmwareChunksReadyForUpdate} = require('./firmware/firmwareHandler')
const {
    addToQueue,
    handleQueue,
    queueCommandCompletedHandler,
    repeatLastRequest
  } = require('./firmware/queue');
const { stat } = require('original-fs');
const { ContactlessOutlined } = require('@material-ui/icons');
const { Console } = require('console');

const listPorts = (onPortOpen,onPortClose,onPortError,onPortData) => {
    SerialPort.list().then((ports) => {
        console.log(ports)
        ports.forEach(function(port) {
            if(port.productId && (port.productId.startsWith('1A9A',0) || port.productId.startsWith('1a9a',0))) {

                PORT1 = new SerialPort(port.path, { autoOpen: true })
                PORT1.on('open', data => { onPortOpen(data) })
                PORT1.on('close', data => { PORT1 = null; onPortClose(data)})
                PORT1.on('error', data => { PORT1.close(); onPortError(data)})
                PORT1.on('data', data => {onPortData(data)})
            }
        });
    })
}


const onPortOpenedBeforeBoot = (data) => {
    getBootloaderVersion(true,PORT1);
}

const onCloseReceivedBeforeBoot = (data) => {
    console.log({CLOSE_BEFORE_BOOT: data})
    setTimeout(() => {
        listPorts(onPortOpenedAfterBoot,onCloseReceivedAfterBoot,onErrorReceivedAfterBoot,onDataReceivedAfterBoot)
    },2000)
}
const onErrorReceivedBeforeBoot = (data) => { console.log({ERROR: data})}

const onDataReceivedBeforeBoot = (data) => {
    const buffer = Buffer.from(data);

    console.log(buffer)

    const bufferLength = buffer.length;

    if (bufferLength < 5 || buffer[0] !== 0) { return emitter.emit('unknownStructureDataReceived', { data: buffer});}

    const dataLength = buffer.readUInt16BE(3);

    if (bufferLength !== 5 + dataLength + 1) {
      return emitter.emit('unknownStructureDataReceived', {
        data: buffer
      });
    }
    const dataBuf = data.slice(5, -1);

    switch (data[1]) {

      case costants.WIFI_GET_BOOTLOADER_VERSION:
         console.log(converters.convertBufferToBootloaderVersion(dataBuf))
         canUpdate = converters.convertBufferToBootloaderVersion(dataBuf).canUpdateFirmware
         if(frontendSocket) frontendSocket.emit('update_allowed',{canUpdate})
         /* if(canUpdate) {
          setTimeout(() => {
             sendBootCommand(true,PORT1);
             bootRequested = true;
          },2000)
        } */
         return;
    }
}


const onPortOpenedAfterBoot = (data) => {
    inBoot = true
    setTimeout(() => {
        updateFirmware()
    },1000)

}


const onCloseReceivedAfterBoot = (data) => {
    console.log({CLOSE_AFTER_BOOT: data})


    setTimeout(() => {

      startTwattUpdate()
      //if(frontendSocket) frontendSocket.emit('update_completed',{})
    },5000)


}
const onErrorReceivedAfterBoot = (data) => { console.log({ERROR: data})}

const onDataReceivedAfterBoot = (data) => {

    //console.log({resp: data})
    const bufS = Buffer.from([0xAA,0xBB,0xCC,0xDD]).toString('hex');
    const dataS = Buffer.from(data).toString('hex');

    if(bufS === dataS) {
        queueCommandCompletedHandler()
    }
    else {
        repeatLastRequest(PORT1)
    }

}

const updateFirmware = () => {
    //const firmwareChunks = getFirmwareChunksReadyForUpdate('./firmware/firmware37.era')
    //const firmwareChunks = getFirmwareChunksReadyForUpdate(path.join(__dirname, 'firmware38.era'))
    const firmwareChunks = getFirmwareChunksReadyForUpdate(path.join(__dirname, 'firmware41.era'))

    //fs.writeFileSync('./firmware.txt', JSON.stringify(firmwareChunks))

    addToQueue('sendUpdateStartCommand',{index: null,chunk: null});

    firmwareChunks.forEach((el) => {
        addToQueue('sendPacketCommand',{index: el.index,chunk: el.chunk});
    })

    addToQueue('sendUpdateResetCommand',{index: null,chunk: null});

}


let PORT1 = null;
let inBoot = false;


listPorts(onPortOpenedBeforeBoot,onCloseReceivedBeforeBoot,onErrorReceivedBeforeBoot,onDataReceivedBeforeBoot)

setInterval(() => {

    handleQueue(PORT1,reportStatus)

},200)


let parser = null
let speedWattTable = []
let getTableIndex = 0;
const USB_GET_WATT_TABLE = 0x11;
const USB_SET_WATT_TABLE = 0x10;

const getWattSpeedTable = Buffer.alloc(7).fill(Buffer.from([0xd0, 0xf1, 0, 8, USB_GET_WATT_TABLE]))
const setWattSpeedTable = Buffer.alloc(7).fill(Buffer.from([0xd0, 0xf1, 1, 8, USB_SET_WATT_TABLE]))

const signBuffer = buffer => {
  const bLength = buffer.length;
  let signedBuffer = Buffer.allocUnsafe(bLength + 1);
  buffer.copy(signedBuffer);
  signedBuffer[bLength] = signedBuffer[0];
  for (let i = 1; i < bLength; i++) {
    signedBuffer[bLength] ^= signedBuffer[i];
  }
  return signedBuffer;
};

const getWattTable = (index,port) => {
  let buffer = Buffer.from(getWattSpeedTable);
  buffer.writeUInt8(index >> 8,5);
  buffer.writeUInt8(index,6);
  return port.write(signBuffer(buffer))
}

const setWattTable = (port,b,index) => {

  console.log({b,index})
  let buffer = Buffer.from(setWattSpeedTable);
  buffer.writeUInt8(index >> 8,5);
  buffer.writeUInt8(index,6);
  const buff = Buffer.concat([Buffer.from(buffer),b])
  return port.write(signBuffer(buff))
}

let size = 256
let size2 = 256
let sizeTAB = 20100
let wifi = 0
let wifi2 = 0
let USERTW = []

const handleRXTwatt = (data) => {
    if(wifi >= sizeTAB) return
    console.log(data)

    if ((wifi + 256) > sizeTAB) size = sizeTAB - wifi;

    if(wifi === 0) {
        const newBuff = Buffer.from(data.slice(5,-1))

        /* newBuff.writeUint16LE(1,8)
        newBuff.writeUint16LE(2,12)
        newBuff.writeUint16LE(3,16)
        newBuff.writeUint16LE(4,20)
        newBuff.writeUint16LE(5,24)
        newBuff.writeUint16LE(6,28)
        newBuff.writeUint16LE(7,32)
        newBuff.writeUint16LE(8,36)
        newBuff.writeUint16LE(9,40)
        newBuff.writeUint16LE(10,44)
        newBuff.writeUint16LE(11,48)
        newBuff.writeUint16LE(12,52)
        newBuff.writeUint16LE(13,56)
        newBuff.writeUint16LE(14,60)
        newBuff.writeUint16LE(15,64) */
        newBuff.writeUint16LE(12,8)
        newBuff.writeUint16LE(14,12)
        newBuff.writeUint16LE(17,16)
        newBuff.writeUint16LE(20,20)
        newBuff.writeUint16LE(22,24)
        newBuff.writeUint16LE(25,28)
        newBuff.writeUint16LE(27,32)
        newBuff.writeUint16LE(29,36)
        newBuff.writeUint16LE(33,40)
        newBuff.writeUint16LE(35,44)
        newBuff.writeUint16LE(39,48)
        newBuff.writeUint16LE(41,52)
        newBuff.writeUint16LE(44,56)
        newBuff.writeUint16LE(50,60)
        newBuff.writeUint16LE(55,64)

        USERTW.push(newBuff)
    }
    else {
        USERTW.push(data.slice(5,-1))
    }

    wifi += size;
    faseAggiornamento = "2/3"
    reportStatus(((wifi/sizeTAB)*100).toFixed(2))
    if(wifi < sizeTAB ) setTimeout(() => {getWattTable(wifi / 256,PORT1)},200)
    else {
        //console.log(USERTW)
        wifi = 0;
        console.log('RX END.')
        //PORT1.unpipe(parser)
        console.log('TX WATT TABLE')
        setTimeout(() => {setWattTable(PORT1, Buffer.from(USERTW[wifi / 256]),wifi / 256)},200)

    }

}



const handleTXTwatt = (data) => {
    console.log({data})
    if ((wifi2 + 256) > sizeTAB) size2 = sizeTAB - wifi2;

    wifi2 += size2;
    faseAggiornamento = "3/3"
    reportStatus(((wifi2/sizeTAB)*100).toFixed(2))
    console.log({wifi2})
    if(wifi2 < sizeTAB ) setTimeout(() => {setWattTable(PORT1, Buffer.from(USERTW[wifi2 / 256]),wifi2 / 256)},200)
    else {
        console.log("end")
        wifi2 = 0;
        PORT1.close();

    }


}


const listPortsTwattUpdate = ({onPortOpen = () => {},onPortClose = () => {},onPortError = () => {},onPortData = () => {}}) => {


  SerialPort.list().then((ports) => {
      console.log({PORTS_AFTER_FW_UPD: ports})
      ports.forEach(function(port) {
          if(port.productId && (port.productId.startsWith('1A9A',0) || port.productId.startsWith('1a9a',0))) {

              //PORT1 = new SerialPort({path: port.path, autoOpen: true,baudRate: 115200 })
              PORT1 = new SerialPort(port.path, { autoOpen: true })
              PORT1.on('open', data => { console.log({OPEN: data}); getWattTable(0,PORT1) })
              PORT1.on('close', data => { PORT1 = null;

                setTimeout(() => {
                  if(frontendSocket) frontendSocket.emit('update_completed',{})
                },5000)

              })
              PORT1.on('error', data => { PORT1.close(); onPortError(data)})
              PORT1.on('data', data => {onPortData(data)})
              parser = PORT1.pipe(new ByteLength({ length: 262 }))
              parser.on('data', (data) => { handleRXTwatt(data) })

          }
      });
  })
}

const startTwattUpdate = () => {

  listPortsTwattUpdate({onPortData: (data) => {
    console.log({PORT_DATA: data})
    if(data[1] === USB_SET_WATT_TABLE) {
        handleTXTwatt(data)
    }

  }})

}
